@@ -4,6 +4,9 @@
 .MatrixEnv <- new.env(parent=emptyenv())
 ## as long as it's small, no 'hash = TRUE'
 
+.chm_common <- new.env(parent = emptyenv())
+## environment in which to store some settings from cholmod_common
+
 ## A wrapper for now [as long as  'methods' has no *exported* version]:
 .M.classEnv <- function(Class) methods:::.classEnv(Class)
 
@@ -33,11 +36,17 @@
     ## This is formally identical to the base definition, but should use the
     ## generic kronecker
     assignInNamespace("%x%", function (X, Y) kronecker(X, Y), ns = "base")
+    ## similar for det() calling determinant() which we make S4-generic:
+    assignInNamespace("det", function (x, ...) {
+	z <- determinant(x, logarithm = TRUE, ...)
+	c(z$sign * exp(z$modulus))
+    }, ns = "base")
 
     ## Hack needed, as C-level  eval / findFun seems not to work with
     ## loaded & non-attached Matrix:
     assignInNamespace(".M.classEnv", .M.classEnv, ns = "base")
 
+    .Call(CHM_set_common_env, .chm_common)
 }
 
 ## Instead, simply re-assign the [cr]bind()s which are recursively
