@@ -6,6 +6,90 @@ Rboolean isValid_Csparse(SEXP x); /* -> Csparse.c */
 cholmod_common c;
 /* cholmod_common cl; */
 
+SEXP chm_common_env;
+
+void CHM_store_common() {
+    SEXP rho = chm_common_env;
+    defineVar(install("dbound"), ScalarReal(c.dbound), rho);
+    defineVar(install("grow0"), ScalarReal(c.grow0), rho);
+    defineVar(install("grow1"), ScalarReal(c.grow1), rho);
+    defineVar(install("grow2"), ScalarInteger(c.grow2), rho);
+    defineVar(install("maxrank"), ScalarInteger(c.maxrank), rho);
+    defineVar(install("supernodal_switch"),
+	      ScalarReal(c.supernodal_switch), rho);
+    defineVar(install("supernodal"),
+	      ScalarInteger(c.supernodal), rho);
+    defineVar(install("final_asis"),
+	      ScalarLogical(c.final_asis), rho);
+    defineVar(install("final_super"),
+	      ScalarLogical(c.final_super), rho);
+    defineVar(install("final_ll"),
+	      ScalarLogical(c.final_ll), rho);
+    defineVar(install("final_pack"),
+	      ScalarLogical(c.final_pack), rho);
+    defineVar(install("final_monotonic"),
+	      ScalarLogical(c.final_monotonic), rho);
+    defineVar(install("final_resymbol"),
+	      ScalarLogical(c.final_resymbol), rho);
+    defineVar(install("prefer_zomplex"),
+	      ScalarLogical(c.prefer_zomplex), rho);
+    defineVar(install("prefer_upper"),
+	      ScalarLogical(c.prefer_upper), rho);
+    defineVar(install("quick_return_if_not_posdef"),
+	      ScalarLogical(c.quick_return_if_not_posdef), rho);
+    defineVar(install("nmethods"), ScalarInteger(c.nmethods), rho);
+    defineVar(install("m0.ord"),
+	      ScalarInteger(c.method[0].ordering), rho);
+    defineVar(install("postorder"), ScalarLogical(c.postorder), rho);
+}
+
+void CHM_restore_common() {
+    SEXP rho = chm_common_env;
+    c.dbound = asReal(findVarInFrame(rho, install("dbound")));
+    c.grow0 = asReal(findVarInFrame(rho, install("grow0")));
+    c.grow1 = asReal(findVarInFrame(rho, install("grow1")));
+    c.grow2 = asInteger(findVarInFrame(rho, install("grow2")));
+    c.maxrank =
+	asInteger(findVarInFrame(rho, install("maxrank")));
+    c.supernodal_switch =
+	asReal(findVarInFrame(rho, install("supernodal_switch")));
+    c.supernodal =
+	asLogical(findVarInFrame(rho, install("supernodal")));
+    c.final_asis =
+	asLogical(findVarInFrame(rho, install("final_asis")));
+    c.final_super =
+	asLogical(findVarInFrame(rho, install("final_super")));
+    c.final_ll =
+	asLogical(findVarInFrame(rho, install("final_ll")));
+    c.final_pack =
+	asLogical(findVarInFrame(rho, install("final_pack")));
+    c.final_monotonic =
+	asLogical(findVarInFrame(rho, install("final_monotonic")));
+    c.final_resymbol =
+	asLogical(findVarInFrame(rho, install("final_resymbol")));
+    c.prefer_zomplex =
+	asLogical(findVarInFrame(rho, install("prefer_zomplex")));
+    c.prefer_upper =
+	asLogical(findVarInFrame(rho, install("prefer_upper")));
+    c.quick_return_if_not_posdef =
+	asLogical(findVarInFrame(rho,
+				 install("quick_return_if_not_posdef")));
+    c.nmethods =
+	asInteger(findVarInFrame(rho, install("nmethods")));
+    c.method[0].ordering =
+	asInteger(findVarInFrame(rho, install("m0.ord")));
+    c.postorder =
+	asLogical(findVarInFrame(rho, install("postorder")));
+}
+
+SEXP CHM_set_common_env(SEXP rho) {
+    if (!isEnvironment(rho))
+	error(_("Argument rho must be an environment"));
+    chm_common_env = rho;
+    CHM_store_common();
+    return R_NilValue;
+}
+
 static int stype(int ctype, SEXP x)
 {
     if ((ctype % 3) == 1) return (*uplo_P(x) == 'U') ? 1 : -1;
@@ -26,13 +110,27 @@ static int xtype(int ctype)
     return -1;
 }
 
+/* coerce a vector to REAL and copy the result to freshly R_alloc'd memory */
+static void *RallocedREAL(SEXP x)
+{
+    SEXP rx = PROTECT(coerceVector(x, REALSXP));
+    int lx = LENGTH(rx);
+    /* We over-allocate the memory chunk so that it is never NULL. */
+    /* The CHOLMOD code checks for a NULL pointer even in the length-0 case. */
+    double *ans = Memcpy((double*)R_alloc(lx + 1, sizeof(double)),
+			 REAL(rx), lx);
+    UNPROTECT(1);
+    return (void*)ans;
+}
+
+
 static void *xpt(int ctype, SEXP x)
 {
     switch(ctype / 3) {
     case 0: /* "d" */
 	return (void *) REAL(GET_SLOT(x, Matrix_xSym));
     case 1: /* "l" */
-	return (void *) REAL(coerceVector(GET_SLOT(x, Matrix_xSym), REALSXP));
+	return RallocedREAL(GET_SLOT(x, Matrix_xSym));
     case 2: /* "n" */
 	return (void *) NULL;
     case 3: /* "z" */
@@ -55,6 +153,9 @@ Rboolean check_sorted_chm(CHM_SP A)
     return TRUE;
 }
 
+/**
+   Copy cholmod_sparse, to an R_alloc()ed version of it
+ */
 static void chm2Ralloc(CHM_SP dest, CHM_SP src)
 {
     int np1, nnz;
@@ -65,12 +166,34 @@ static void chm2Ralloc(CHM_SP dest, CHM_SP src)
     /* R_alloc the vector storage for dest and copy the contents from src */
     np1 = src->ncol + 1;
     nnz = (int) cholmod_l_nnz(src, &c);
-    dest->p = (void*) Memcpy((   int*)R_alloc(sizeof(   int), np1),
-			     (   int*)(src->p), np1);
-    dest->i = (void*) Memcpy((   int*)R_alloc(sizeof(   int), nnz),
-			     (   int*)(src->i), nnz);
-    dest->x = (void*) Memcpy((double*)R_alloc(sizeof(double), nnz),
-			     (double*)(src->x), nnz);
+    dest->p = (void*) Memcpy((int*)R_alloc(sizeof(int), np1),
+			     (int*)(src->p), np1);
+    dest->i = (void*) Memcpy((int*)R_alloc(sizeof(int), nnz),
+			     (int*)(src->i), nnz);
+    if(src->xtype)
+	dest->x = (void*) Memcpy((double*)R_alloc(sizeof(double), nnz),
+				 (double*)(src->x), nnz);
+}
+
+/**
+   Copy cholmod_triplet to an R_alloc()ed version of it
+ */
+static void chTr2Ralloc(CHM_TR dest, CHM_TR src)
+{
+    int np1, nnz;
+
+    /* copy all the (non-pointer) characteristics of src to dest */
+    memcpy(dest, src, sizeof(cholmod_triplet));
+
+    /* R_alloc the vector storage for dest and copy the contents from src */
+    nnz = src->nnz;
+    dest->i = (void*) Memcpy((int*)R_alloc(sizeof(int), nnz),
+			     (int*)(src->i), nnz);
+    dest->j = (void*) Memcpy((int*)R_alloc(sizeof(int), nnz),
+			     (int*)(src->j), nnz);
+    if(src->xtype)
+	dest->x = (void*) Memcpy((double*)R_alloc(sizeof(double), nnz),
+				 (double*)(src->x), nnz);
 }
 
 /**
@@ -78,8 +201,8 @@ static void chm2Ralloc(CHM_SP dest, CHM_SP src)
  * elements accordingly. Note that later changes to the contents of
  * ans will change the contents of the SEXP.
  *
- * In most cases this function is called through the macro
- * AS_CHM_SP().  It is unusual to call it directly.
+ * In most cases this function is called through the macros
+ * AS_CHM_SP() or AS_CHM_SP__().  It is unusual to call it directly.
  *
  * @param ans a CHM_SP pointer
  * @param x pointer to an object that inherits from CsparseMatrix
@@ -89,9 +212,11 @@ static void chm2Ralloc(CHM_SP dest, CHM_SP src)
  *  should they be sorted in place?  If the i and x slots are pointers
  *  to an input SEXP they should not be modified.
  *
- * @return ans containing pointers to the slots of x.
+ * @return ans containing pointers to the slots of x, *unless*
+ *	check_Udiag and x is unitriangular.
  */
-CHM_SP as_cholmod_sparse(CHM_SP ans, SEXP x, Rboolean check_Udiag, Rboolean sort_in_place)
+CHM_SP as_cholmod_sparse(CHM_SP ans, SEXP x,
+			 Rboolean check_Udiag, Rboolean sort_in_place)
 {
     char *valid[] = {"dgCMatrix", "dsCMatrix", "dtCMatrix",
 		     "lgCMatrix", "lsCMatrix", "ltCMatrix",
@@ -100,7 +225,6 @@ CHM_SP as_cholmod_sparse(CHM_SP ans, SEXP x, Rboolean check_Udiag, Rboolean sort
 		     ""};
     int *dims = INTEGER(GET_SLOT(x, Matrix_DimSym)),
 	ctype = Matrix_check_class_etc(x, valid);
-
     SEXP islot = GET_SLOT(x, Matrix_iSym);
 
     if (ctype < 0) error(_("invalid class of object to as_cholmod_sparse"));
@@ -117,9 +241,11 @@ CHM_SP as_cholmod_sparse(CHM_SP ans, SEXP x, Rboolean check_Udiag, Rboolean sort
 				/* dimensions and nzmax */
     ans->nrow = dims[0];
     ans->ncol = dims[1];
-    ans->nzmax = (size_t) ((int *)ans->p)[dims[1]];
-    /* == LENGTH(islot) only if the i-slot is not over-allocated */
-
+    /* Allow for over-allocation of the i and x slots.  Needed for
+     * sparse X form in lme4.  Right now it looks too difficult to
+     * check for the length of the x slot, because of the xpt
+     * utility, but the lengths of x and i should agree. */
+    ans->nzmax = LENGTH(islot);
 				/* values depending on ctype */
     ans->x = xpt(ctype, x);
     ans->stype = stype(ctype, x);
@@ -141,7 +267,7 @@ CHM_SP as_cholmod_sparse(CHM_SP ans, SEXP x, Rboolean check_Udiag, Rboolean sort
 #ifdef DEBUG_Matrix
 	    /* This "triggers" exactly for return values of dtCMatrix_sparse_solve():*/
 	    /* Don't want to translate this: want it report */
-	    Rprintf("Note: as_cholmod_l_sparse() needed cholmod_l_sort()ing\n");
+	    Rprintf("Note: as_cholmod_sparse() needed cholmod_l_sort()ing\n");
 #endif
 	    chm2Ralloc(ans, tmp);
 	    cholmod_l_free_sparse(&tmp, &c);
@@ -153,6 +279,9 @@ CHM_SP as_cholmod_sparse(CHM_SP ans, SEXP x, Rboolean check_Udiag, Rboolean sort
 	CHM_SP eye = cholmod_l_speye(ans->nrow, ans->ncol, ans->xtype, &c);
 	CHM_SP tmp = cholmod_l_add(ans, eye, one, one, TRUE, TRUE, &c);
 
+#ifdef DEBUG_Matrix_verbose /* quite often, e.g. in ../tests/indexing.R */
+	Rprintf("Note: as_cholmod_sparse(<ctype=%d>) - diagU2N\n", ctype);
+#endif
 	chm2Ralloc(ans, tmp);
 	cholmod_l_free_sparse(&tmp, &c);
 	cholmod_l_free_sparse(&eye, &c);
@@ -168,7 +297,7 @@ CHM_SP as_cholmod_sparse(CHM_SP ans, SEXP x, Rboolean check_Udiag, Rboolean sort
  * Copy the contents of a to an appropriate CsparseMatrix object and,
  * optionally, free a or free both a and its the pointers to its contents.
  *
- * @param a matrix to be converted
+ * @param a  (cholmod_sparse) matrix to be converted
  * @param dofree 0 - don't free a; > 0 cholmod_l_free a; < 0 Free a
  * @param uploT 0 - not triangular; > 0 upper triangular; < 0 lower
  * @param Rkind - vector type to store for a->xtype == CHOLMOD_REAL,
@@ -258,15 +387,16 @@ SEXP chm_sparse_to_SEXP(CHM_SP a, int dofree, int uploT, int Rkind,
  * elements accordingly. Note that later changes to the contents of
  * ans will change the contents of the SEXP.
  *
- * In most cases this function is called through the macro AS_CHM_SP.
- * It is unusual to call it directly.
+ * In most cases this function is called through the macros
+ * AS_CHM_TR() or AS_CHM_TR__().  It is unusual to call it directly.
  *
  * @param ans a CHM_TR pointer
  * @param x pointer to an object that inherits from TsparseMatrix
  * @param check_Udiag boolean - should a check for (and consequent
  *  expansion of) a unit diagonal be performed.
  *
- * @return ans containing pointers to the slots of x.
+ * @return ans containing pointers to the slots of x, *unless*
+ *	check_Udiag and x is unitriangular.
  */
 CHM_TR as_cholmod_triplet(CHM_TR ans, SEXP x, Rboolean check_Udiag)
 {
@@ -275,44 +405,43 @@ CHM_TR as_cholmod_triplet(CHM_TR ans, SEXP x, Rboolean check_Udiag)
 		     "ngTMatrix", "nsTMatrix", "ntTMatrix",
 		     "zgTMatrix", "zsTMatrix", "ztTMatrix",
 		     ""};
-    int *dims, ctype = Matrix_check_class_etc(x, valid), m;
-    SEXP islot;
+    int *dims = INTEGER(GET_SLOT(x, Matrix_DimSym)),
+	ctype = Matrix_check_class_etc(x, valid);
+    SEXP islot = GET_SLOT(x, Matrix_iSym);
+    int m = LENGTH(islot);
     Rboolean do_Udiag = (check_Udiag && ctype % 3 == 2 && (*diag_P(x) == 'U'));
-
     if (ctype < 0) error(_("invalid class of object to as_cholmod_triplet"));
+
     memset(ans, 0, sizeof(cholmod_triplet)); /* zero the struct */
 
     ans->itype = CHOLMOD_LONG;	/* characteristics of the system */
     ans->dtype = CHOLMOD_DOUBLE;
-    ans->x = ans->z = (void *) NULL;
-				/* dimensions and nzmax */
-    dims = INTEGER(GET_SLOT(x, Matrix_DimSym));
+				/* nzmax, dimensions, types and slots : */
+    ans->nnz = ans->nzmax = m;
     ans->nrow = dims[0];
     ans->ncol = dims[1];
-
-    islot = GET_SLOT(x, Matrix_iSym);
-    m = LENGTH(islot);
-    ans->nnz = ans->nzmax = do_Udiag ? m + dims[0] : m;
-				/* slots always present */
-    ans->i = (void *) INTEGER(islot);
-    ans->j = (void *) INTEGER(GET_SLOT(x, Matrix_jSym));
-
     ans->stype = stype(ctype, x);
     ans->xtype = xtype(ctype);
+    ans->i = (void *) INTEGER(islot);
+    ans->j = (void *) INTEGER(GET_SLOT(x, Matrix_jSym));
     ans->x = xpt(ctype, x);
 
-    if(do_Udiag) {  /* Tsparse_diagU2N(.)  [ ./Tsparse.c ]  "in place" : */
+    if(do_Udiag) {
+	/* diagU2N(.) "in place", similarly to Tsparse_diagU2N [./Tsparse.c]
+	   (but without new SEXP): */
 	int k = m + dims[0];
+	CHM_TR tmp = cholmod_l_copy_triplet(ans, &c);
 	int *a_i, *a_j;
 
-	/* TODO? instead of reallocating, don't do the 2nd part of AS_CHM_COMMON()
-	 * ---- above, and allocate to correct length + Memcpy() here, as in
-	 * Tsparse_diagU2N() */
-	if(cholmod_l_reallocate_triplet((size_t) k, ans, &c))
-	    error(_("as_cholmod_l_triplet(): could not reallocate for internal diagU2N()"
+	if(!cholmod_l_reallocate_triplet((size_t) k, tmp, &c))
+	    error(_("as_cholmod_triplet(): could not reallocate for internal diagU2N()"
 		      ));
-	a_i = ans->i;
-	a_j = ans->j;
+
+	/* TODO? instead of copy_triplet() & reallocate_triplet()
+	 * ---- allocate to correct length + Memcpy() here, as in
+	 * Tsparse_diagU2N() & chTr2Ralloc() below */
+	a_i = tmp->i;
+	a_j = tmp->j;
 	/* add (@i, @j)[k+m] = k, @x[k+m] = 1.   for k = 0,..,(n-1) */
 	for(k=0; k < dims[0]; k++) {
 	    a_i[k+m] = k;
@@ -320,25 +449,28 @@ CHM_TR as_cholmod_triplet(CHM_TR ans, SEXP x, Rboolean check_Udiag)
 
 	    switch(ctype / 3) {
 	    case 0: { /* "d" */
-		double *a_x = ans->x;
+		double *a_x = tmp->x;
 		a_x[k+m] = 1.;
 		break;
 	    }
 	    case 1: { /* "l" */
-		int *a_x = ans->x;
+		int *a_x = tmp->x;
 		a_x[k+m] = 1;
 		break;
 	    }
 	    case 2: /* "n" */
 		break;
 	    case 3: { /* "z" */
-		double *a_x = ans->x;
+		double *a_x = tmp->x;
 		a_x[2*(k+m)  ] = 1.;
 		a_x[2*(k+m)+1] = 0.;
 		break;
 	    }
 	    }
-	}
+	} /* for(k) */
+
+	chTr2Ralloc(ans, tmp);
+	cholmod_l_free_triplet(&tmp, &c);
 
     } /* else :
        * NOTE: if(*diag_P(x) == 'U'), the diagonal is lost (!);
@@ -494,9 +626,7 @@ CHM_DN as_cholmod_dense(CHM_DN ans, SEXP x)
 
     case 1: /* "l" */
 	ans->xtype = CHOLMOD_REAL;
-	ans->x =
-	    (void *) REAL(coerceVector((ctype % 2) ? GET_SLOT(x, Matrix_xSym) : x,
-				       REALSXP));
+	ans->x = RallocedREAL((ctype % 2) ? GET_SLOT(x, Matrix_xSym) : x);
 	break;
     case 2: /* "n" */
 	ans->xtype = CHOLMOD_PATTERN;
@@ -525,9 +655,7 @@ CHM_DN as_cholmod_x_dense(cholmod_dense *ans, SEXP x)
     case 1: /* "l" */
     case 2: /* "n" (no NA in 'x', but *has* 'x' slot => treat as "l" */
 	ans->xtype = CHOLMOD_REAL;
-	ans->x =
-	    (void *) REAL(coerceVector((ctype % 2) ? GET_SLOT(x, Matrix_xSym) : x,
-				       REALSXP));
+	ans->x = RallocedREAL((ctype % 2) ? GET_SLOT(x, Matrix_xSym) : x);
 	break;
 
     _AS_cholmod_dense_2;
@@ -538,14 +666,16 @@ CHM_DN as_cholmod_x_dense(cholmod_dense *ans, SEXP x)
 
 void R_cholmod_error(int status, const char *file, int line, const char *message)
 {
+    CHM_restore_common(); /* restore any setting that may have been changed */
+
+/* NB: keep in sync with M_R_cholmod_error(), ../inst/include/Matrix_stubs.c */
+
     /* From CHOLMOD/Include/cholmod_core.h : ...status values.
        zero means success, negative means a fatal error, positive is a warning.
     */
 #ifndef R_CHOLMOD_ALWAYS_ERROR
     if(status < 0) {
 #endif
-	cholmod_l_defaults(&c);/* <--- restore defaults,
-				* as we will not be able to .. */
 	error(_("Cholmod error '%s' at file:%s, line %d"), message, file, line);
 #ifndef R_CHOLMOD_ALWAYS_ERROR
     }
@@ -577,13 +707,13 @@ int R_cholmod_printf(const char* fmt, ...)
  */
 int R_cholmod_l_start(CHM_CM cl)
 {
+/* NB: keep in sync with M_R_cholmod_l_start(), ../inst/include/Matrix_stubs.c */
     int res;
     if (!(res = cholmod_l_start(cl)))
 	error(_("Unable to initialize cholmod_l: error code %d"), res);
     cl->print_function = R_cholmod_printf; /* Rprintf gives warning */
-    /* Since we provide an error handler, it may not be a good idea to allow CHOLMOD printing,
-     * because that's not easily suppressed on the R level :
-     * Hence consider, at least temporarily *  cl->print_function = NULL;
+    /* Since we provide an error handler, it may not be a good idea to
+     * allow CHOLMOD printing.
      * FIXME? alternative {slightly more efficient}:
      * do *compile* CHOLMOD with '#define NPRINT'  */
     cl->print_function = NULL;
